
################################
## Drawing random quantities  ##
##    and identifying the     ##
## characteristics of objects ##
################################

## Being able to check characteristics of objects
## is useful to make sure your code does what you
## think it does.
## i.e. can you tell if a given object is a matrix?

## Here we draw 1000 draws from a normal
x <- rnorm(1000,mean=10,sd=1)
## Let's confirm that it's a vector of length 1000
str(x)
length(x)

## There were questions about what rnorm() is.
## Type ?rnorm to see help file
## Remember we've been using qnorm() to get
## the z-score of interest for confidence intervals
## and hypothesis tests
qnorm(.975)
## pnorm() gives the cumulative area under the curvey up
## until the z-value you enter (it assumes mean 0
## and standard deviation 1 unless you specify otherwise)
pnorm(1.96)
## To get the area to the right of the point, we can either
## take the complement or use the lower.tail=F option
1-pnorm(1.96)
pnorm(1.96,lower.tail=F)

## Suppose we want a matrix with 100 rows and 10 columns,
## where each column holds 100 draws from a normal with
## mean equal to the column number (1,2,...,10)
## and standard deviation 1

## Set the seed to the zip code for East Rochester, NY,
## or whatever zip code or series of digits you prefer
set.seed(14445)
## Initialize a matrix with missing values everywhere
## and the desired dimensions
X <- matrix(NA, nrow=100, ncol=10)
## Run a for-loop to fill in the values
for (i in 1:ncol(X)) {
  ## Draw a vector of 100 realizations from a normal
  ## with mean i and standard deviation 1
  draw <- rnorm(nrow(X), mean=i, sd=1)
  ## Store that vector in the i-th column of the matrix X
  X[,i] <- draw
}

## Check that we got what we thought
## Dimensions should be 100 x 10
dim(X)
## Lenth should be 100 x 10 = 1,000
length(X)
## str() will also tell us about this object
str(X)

######################
## Repetitive code, ##
## loops, and apply ##
######################

## What if we didn't know the true data generating process,
## and we wanted to estimate the means of the columns of X?

## We could do this tediously, column by column
mean_col1 <- mean(X[,1])
mean_col2 <- mean(X[,2])

## But when something feels repetitive in R, there's
## generally an easier way!

## Let's initialize a vector of means of length 10
means <- numeric(10)
## could also use
## means <- rep(NA,10)
## Loop through the numbers 1 through ncol(X)=10
## and calculate the mean for the column at each iteration,
## storing the result in the means vector.
for (i in 1:ncol(X)) {
  means[i] <- mean(X[,i])
}

## We could do that easier with apply()
## Type ?apply to see how this works
## We apply the function mean (FUN = mean),
## over columns (MAR=2),
## of the matrix X (X=X)

## This does the function to each column of X
## and returns a vector
means.from.apply <- apply(
  X = X, MAR = 2, FUN = mean
)
## Let's look at that
means.from.apply

## sapply() does a function to all elements of a list
## A data frame is a list of columns, where each column
## must have the same length. Let's make X a data frame.
X.data.frame <- data.frame(X)
## See that the class is now data.frame
class(X.data.frame)
## Take the mean of all the columns
## sapply() will return a vector of the results
sapply(X.data.frame,mean)
## lapply() does the same thing but returns
## a list of the results.
lapply(X.data.frame,mean)

########################
##   Why would we     ##
##  ever use a list?  ##
## (student question) ##
########################

## Someone asked why we'd ever use a list.
## A list is good when you want to store a list of
## objects all in one place, but they might all have
## different lengths and different data types.
## The list below has three elements, all with
## different lengths, where the first two are numeric
## and the third is character.
example.list <- list(first.element=c(1,2,3),
                     second.element=2,
                     third.element=c("Hello","Goodbye"))
## Just like a data frame, we can get an element of a list
## with the $ operator
example.list$first.element

## We can apply the class() function to all elements of the list
## and see that the first two are numeric and the third
## is character
sapply(example.list, class)
## We can also see the length of each element
sapply(example.list, length)

## If the elements of your list all have the same length,
## you might as well just use a data frame!

#############################
## Repetitive code and the ##
##    table() function     ##
#############################

## A lot of people had repetitive code
## to find the number of 1s, 2s, etc. as
## the last digit in election results
## on problem set 2.

## I just wanted to briefly review how
## this can be done more easily with table()

## Take a sample of digits from 0 to 9
numbers <- sample(0:9,100,replace=T)

## The long and repetitive way would be
number.of.ones <- sum(numbers==1)
number.of.twos <- sum(numbers==2)
## That's what many people did on the homework.

## The easy way is with the table() function
table(numbers)

## If you're doing something repetitive, ask
## us or Google if there is an easier way!


